# -*- coding:utf-8; eval: (blacken-mode) -*-
import os
import numpy as np
import functools

import matplotlib.pyplot as plt
import matplotlib.cm as cm
import matplotlib.colors as colors

__all__ = [
    "plot_Delta_J_2d",
    "ComplexColorMesh",
    "pcolormesh_complex",
    "cmap_hue_cyclic",
    "cmap_huegray_r",
]


def plot_Delta_J_2d(x, y, Delta, J, ax=None):
    if ax is None:
        ax = plt.gca()

    m = pcolormesh_complex(x, y, Delta, ax=ax)
    I = J.real
    xc = (x[1:] + x[:-1]) / 2
    yc = (y[1:] + y[:-1]) / 2
    Ix = I[0, :-1, :-1, 0].T
    Iy = I[1, :-1, :-1, 0].T
    lw = np.hypot(Ix, Iy)
    lw = 6 * (lw / max(1.0, lw.max())) ** 0.5

    ax.streamplot(xc, yc, Ix, Iy, color=(0.5, 0.5, 0.5), linewidth=lw)
    ax.set_xlabel(r"$x$")
    ax.set_ylabel(r"$y$")
    plt.colorbar(m.phase, label=r"$\varphi\,/ \pi$", ax=ax)
    plt.colorbar(m.amplitude, label=r"$|\Delta|$", ax=ax)
    ax.set_aspect("equal")

    return m


class ComplexColorMesh:
    def __init__(self, phase, amplitude):
        self.phase = phase
        self.amplitude = amplitude

    @classmethod
    def _get_phase_alpha(cls, data):
        abs_max = np.abs(data).max()
        phase = np.angle(data).transpose(1, 0) / np.pi
        alpha = np.abs(data).transpose(1, 0) / abs_max
        return phase, alpha, abs_max

    def set_data(self, data):
        phase, alpha, abs_max = self._get_phase_alpha(data)
        self.phase.set_data(phase)
        self.phase.set_alpha(alpha)
        self.amplitude.set_clim(0, abs_max)

    def draw(self):
        return self.phase.draw()


def pcolormesh_complex(x, y, data, ax=None):
    nx = data.shape[0]
    ny = data.shape[1]
    dx = np.median(np.diff(x))
    dy = np.median(np.diff(y))

    if len(x) != nx or len(y) != ny:
        raise ValueError("Invalid coordinates")

    origin = "lower"
    extent = (x[0] - dx / 2, x[-1] + dx / 2, y[0] - dy / 2, y[-1] + dy / 2)
    kw = dict(origin=origin, extent=extent)

    phase, alpha, abs_max = ComplexColorMesh._get_phase_alpha(data)

    if ax is None:
        ax = plt.gca()

    m_phase = ax.imshow(
        phase,
        cmap=cmap_hue_cyclic(),
        vmin=-1,
        vmax=1,
        alpha=alpha,
        **kw,
    )

    ax.set_aspect("auto")

    amp_norm = colors.Normalize(vmin=0, vmax=abs_max)
    m_amplitude = cm.ScalarMappable(amp_norm, cmap=cmap_huegray_r())

    return ComplexColorMesh(m_phase, m_amplitude)


@functools.lru_cache()
def cmap_hue_cyclic():
    from ._plotting_data import data

    return _get_cmap(data)


@functools.lru_cache()
def cmap_huegray_r():
    from ._plotting_data import data_l

    return _get_cmap(np.c_[data_l, data_l, data_l][::-1])


def _get_cmap(data):
    from matplotlib.colors import LinearSegmentedColormap

    data = np.asarray(data)
    x = np.linspace(0, 1, data.shape[0])

    y0_r = np.r_[data[0, 0], data[:-1, 0]]
    y1_r = np.r_[data[1:, 0], data[-1, 0]]

    y0_g = np.r_[data[0, 1], data[:-1, 1]]
    y1_g = np.r_[data[1:, 1], data[-1, 1]]

    y0_b = np.r_[data[0, 2], data[:-1, 2]]
    y1_b = np.r_[data[1:, 2], data[-1, 2]]

    red = np.c_[x, y0_r, y1_r].tolist()
    green = np.c_[x, y0_g, y1_g].tolist()
    blue = np.c_[x, y0_b, y1_b].tolist()

    return LinearSegmentedColormap("jch", {"red": red, "green": green, "blue": blue})


def _generate():
    import colorspacious

    data = colorspacious.cspace_convert(
        np.c_[63 * np.ones(120), 48 * np.ones(120), 360 * np.linspace(0, 1, 120)],
        "JCh",
        "sRGB1",
    )

    data_l = (
        colorspacious.cspace_convert(
            np.c_[
                63 * np.linspace(1, 1.59, 120),
                0 * np.ones(120),
                0 * np.linspace(0, 1, 120),
            ],
            "JCh",
            "sRGB1",
        )
        .mean(axis=1)
        .clip(0, 1)
    )

    dst = os.path.join(os.path.dirname(__file__), "_plotting_data.py")
    with open(dst, "w") as f:
        f.write(f"data = {data.tolist()}\n" f"data_l = {data_l.tolist()}\n")
